@@ -36,11 +36,19 @@ import dbus.service
 import dbus.glib
 
 from gi import require_version
+require_version('GLib', '2.0')
+from gi.repository import GLib
+
+# set_prgname before importing other modules to show the name in warning
+# messages when import modules are failed. E.g. Gtk.
+GLib.set_application_name('Typing Booster Preferences')
+# This makes gnome-shell load the .desktop file when running under Wayland:
+GLib.set_prgname('ibus-setup-typing-booster')
+
 require_version('Gtk', '3.0')
 from gi.repository import Gtk
 require_version('IBus', '1.0')
 from gi.repository import IBus
-from gi.repository import GLib
 from pkginstall import InstallPkg
 from i18n import DOMAINNAME, _, init as i18n_init
 
@@ -155,8 +163,11 @@ class SetupUI:
         # from that .desktop file. If the .desktop file cannot be
         # found, the second argument of set_wmclass() is shown by
         # gnome-shell in the top bar.
+        #
+        # It only works like this when gnome-shell runs under Xorg
+        # though, under Wayland things are different.
         maindialog.set_wmclass('ibus-setup-typing-booster', 'Typing Booster Preferences')
-        maindialog.show()
+        maindialog.show_all()
 
         name_version = self.builder.get_object("name_version_label")
         name_version.set_markup(
@@ -275,6 +286,30 @@ class SetupUI:
         off_the_record_checkbox.connect(
             'clicked', event_handler.on_off_the_record_checkbox)
 
+        qt_im_module_workaround_checkbox = self.builder.get_object(
+            "qt_im_module_workaround_checkbox")
+        self.qt_im_module_workaround = itb_util.variant_to_value(
+            self.config.get_value(
+                self.config_section, 'qtimmoduleworkaround'))
+        if self.qt_im_module_workaround is None:
+            self.qt_im_module_workaround = False
+        if self.qt_im_module_workaround is True:
+            qt_im_module_workaround_checkbox.set_active(True)
+        qt_im_module_workaround_checkbox.connect(
+            'clicked', event_handler.on_qt_im_module_workaround_checkbox)
+
+        arrow_keys_reopen_preedit_checkbox = self.builder.get_object(
+            "arrow_keys_reopen_preedit_checkbox")
+        self.arrow_keys_reopen_preedit = itb_util.variant_to_value(
+            self.config.get_value(
+                self.config_section, 'arrowkeysreopenpreedit'))
+        if self.arrow_keys_reopen_preedit is None:
+            self.arrow_keys_reopen_preedit = False
+        if self.arrow_keys_reopen_preedit is True:
+            arrow_keys_reopen_preedit_checkbox.set_active(True)
+        arrow_keys_reopen_preedit_checkbox.connect(
+            'clicked', event_handler.on_arrow_keys_reopen_preedit_checkbox)
+
         show_status_info_in_auxiliary_text_checkbox = self.builder.get_object(
             "show_status_info_in_auxiliary_text_checkbox")
         self.show_status_info_in_auxiliary_text = itb_util.variant_to_value(
@@ -784,6 +819,43 @@ class EventHandler:
                 'offtherecord',
                 GLib.Variant.new_boolean(False))
 
+    def on_qt_im_module_workaround_checkbox(self, widget):
+        '''
+        The checkbox whether to use the workaround for the broken
+        implementation of forward_key_event() in the Qt 4/5 input
+        module, has been clicked.
+        '''
+        if widget.get_active():
+            SETUP_UI.qt_im_module_workaround = True
+            SETUP_UI.config.set_value(
+                SETUP_UI.config_section,
+                'qtimmoduleworkaround',
+                GLib.Variant.new_boolean(True))
+        else:
+            SETUP_UI.qt_im_module_workaround = False
+            SETUP_UI.config.set_value(
+                SETUP_UI.config_section,
+                'qtimmoduleworkaround',
+                GLib.Variant.new_boolean(False))
+
+    def on_arrow_keys_reopen_preedit_checkbox(self, widget):
+        '''
+        The checkbox whether arrow keys are allowed to reopen
+        a preëdit, has been clicked.
+        '''
+        if widget.get_active():
+            SETUP_UI.arrow_keys_reopen_preedit = True
+            SETUP_UI.config.set_value(
+                SETUP_UI.config_section,
+                'arrowkeysreopenpreedit',
+                GLib.Variant.new_boolean(True))
+        else:
+            SETUP_UI.arrow_keys_reopen_preedit = False
+            SETUP_UI.config.set_value(
+                SETUP_UI.config_section,
+                'arrowkeysreopenpreedit',
+                GLib.Variant.new_boolean(False))
+
     def on_show_status_info_in_auxiliary_text_checkbox(self, widget):
         '''
         The checkbox whether to show status in the auxiliary text,
